@@ -11,6 +11,7 @@ from django.core.management.base import BaseCommand, CommandError
 from django.conf import settings
 import os
 import re
+import string
 
 
 def get_module_dirpath(module_name):
@@ -45,8 +46,9 @@ class ModulesWriter(object):
     Generates an auto_modules.rst file referencing all the app's automodules
     """
 
-    def __init__(self, filename, doc_title, internal_title, external_title, automodule_options,
-                 excluded_modules, excluded_apps):
+    def __init__(self, docs_root, filename, doc_title, internal_title, external_title,
+                 automodule_options, excluded_modules, excluded_apps):
+        self.docs_root = docs_root
         self.filename = filename
         self.doc_title = doc_title
         self.internal_title = internal_title
@@ -66,7 +68,7 @@ class ModulesWriter(object):
         self.add_section(self.internal_title, self.internal_lines)
         self.add_section(self.external_title, self.external_lines)
 
-        f = open(self.filename, "w+")
+        f = open(os.path.join(self.docs_root, "%s.rst" % self.filename), "w+")
         f.writelines("%s\n" % l for l in self.lines)
         f.close()
 
@@ -75,7 +77,7 @@ class ModulesWriter(object):
         self.lines.extend(sphinx_heading(heading_level, title))
         self.lines.extend(lines)
 
-    def add_to_toc(self, master_doc_path):
+    def add_to_toc(self, master_doc_name):
         """
         Verifies that a "auto_modules" file is in the toctree, and appends it
         otherwise.
@@ -83,15 +85,35 @@ class ModulesWriter(object):
         Note:
             This method writes to the file at master_doc_path!
         """
+        master_doc_path = os.path.join(self.docs_root, "%s.rst" % master_doc_name)
         master_doc = open(master_doc_path, "r")
         master_doc_lines = master_doc.readlines()
 
+        def next_is_blank(lines, i):
+            try:
+                next_is_blank = (lines[i+1].lstrip() == '')
+            except IndexError:
+                return False
+            else:
+                return next_is_blank
+
         if not self.filename in "".join(master_doc_lines):
             # append the new file name to the index.rst
             for i, line in enumerate(master_doc_lines):
                 if ":maxdepth: 2" in line:
-                    master_doc_lines.insert(i + 2, "    %s\n" % self.filename)
                     break
+            # add preceding blank line if needed
+            if not next_is_blank(master_doc_lines, i):
+                master_doc_lines.insert(i+1, "\n")
+            # add auto_modules line
+            master_doc_lines.insert(i+2, "   %s\n" % self.filename)
+            # add following blank line if needed
+            try:
+                following_char = master_doc_lines[i+3][0]
+            except IndexError:
+                following_char = ''
+            if not following_char in string.whitespace:
+                master_doc_lines.insert(i+3, "\n")
         master_doc = open(master_doc_path, "w")
         master_doc.writelines(master_doc_lines)
         master_doc.close()
@@ -196,7 +218,7 @@ class Command(BaseCommand):
         except IndexError:
             docs_root = getattr(settings, "GIZA_DOCS_ROOT", os.path.join(PROJECT_ROOT, "docs"))
 
-        master_doc = getattr(settings, "GIZA_INDEX_DOC", "index.rst")
+        master_doc = getattr(settings, "GIZA_INDEX_DOC", "index")
         filename = getattr(settings, "GIZA_FILENAME", "auto_modules")
         doc_title = getattr(settings, "GIZA_DOC_TITLE", "Python modules")
         internal_title = getattr(settings, "GIZA_INTERNAL_TITLE", "Project Apps")
@@ -217,12 +239,15 @@ class Command(BaseCommand):
         ])
 
         modules_writer = ModulesWriter(
-            filename=os.path.join(docs_root, "%s.rst" % filename),
+            docs_root=docs_root,
+            filename=filename,
             doc_title=doc_title,
+            internal_title=internal_title,
+            external_title=external_title,
             automodule_options=automodule_options,
             excluded_modules=excluded_modules,
             excluded_apps=excluded_apps,
         )
         modules_writer.add_apps(settings.INSTALLED_APPS)
-        modules_writer.add_to_toc(os.path.join(docs_root, master_doc))
+        modules_writer.add_to_toc(master_doc)
         modules_writer.write()
